using System;
using System.Net.Http;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using NwndEntFwkAPI.Domain;

namespace NwndEntFwk
{
    /// <summary>
    /// Helper functions or methods used by the current application.
    /// </summary>
    public sealed class Functions
    {
        private Functions()
        {
        }

        /// <summary>
        /// Removes non-alphanumeric characters other than a Space and Dash.
        /// </summary>
        /// <param name="text">String text that needs to be filtered</param>
        /// <returns>Returns a String.  E.g. 12Abc#$ ef-g will return 12Abc ef-g</returns>
        private static string RemoveSpecialChars(string text)
        {
            Regex regex = new Regex("[^a-zA-Z0-9 -]");
            return regex.Replace(text, "");
        }

        /// <summary>
        /// Used in the ListForeach razor view.
        /// Gets the Start Page shown in the footer (pager) of the ListForeach grid along with page numbers.
        /// </summary>
        /// <param name="currentPage">Page number where the current grid is</param>
        /// <param name="numberOfPagesToShow">Number of pages to show in the pager between the First and Last links</param>
        /// <returns>Start Page in the pager</returns>
        public static int GetPagerStartPage(int currentPage, int numberOfPagesToShow)
        {
            int startPage = 1;

            if (currentPage <= numberOfPagesToShow)
                startPage = 1;
            else if ((currentPage > numberOfPagesToShow) && (currentPage % numberOfPagesToShow == 0))
                startPage = ((currentPage / numberOfPagesToShow) - 1) * numberOfPagesToShow + 1;
            else if (currentPage > numberOfPagesToShow)
                startPage = (currentPage / numberOfPagesToShow) * numberOfPagesToShow + 1;

            return startPage;
        }

        /// <summary>
        /// Used in the ListForeach razor view.
        /// Gets the End Page shown in the footer (pager) of the ListForeach grid along with page numbers.
        /// </summary>
        /// <param name="startPage">Page number where the current grid is</param>
        /// <param name="numberOfPagesToShow">Number of pages to show in the pager between the First and Last links</param>
        /// <param name="totalPages">Total number of pages</param>
        /// <returns>Start Page in the pager</returns>
        public static int GetPagerEndPage(int startPage, int numberOfPagesToShow, int totalPages)
        {
            int endPage = startPage + (numberOfPagesToShow - 1);

            if (endPage >= totalPages)
                return totalPages;
            else
                return startPage + (numberOfPagesToShow - 1);
        }

        /// <summary>
        /// The default Base Address for the Web API project.
        /// </summary>
        /// <returns>E.g. "http://localhost:27230/"</returns>
        public static string GetWebApiBaseAddress()
        {
            return "http://localhost:27230/";
        }

        /// <summary>
        /// Used in the ListForeach razor view pager shown in the footer of the grid.
        /// Gets the number of rows to show per page.
        /// </summary>
        public static int GetGridNumberOfRows()
        {
            return 15;
        }

        /// <summary>
        /// Used in the ListForeach razor view pager shown in the footer of the grid.
        /// Gets the number of pages to show in between the First and Last link.
        /// </summary>
        public static int GetGridNumberOfPagesToShow()
        {
            return 10;
        }

        /// <summary>
        /// Calls a Web API URL using HttpClient GetAsync.
        /// </summary>
        /// <param name="webApiUrl">Web API URL</param>
        /// <returns>Returns a String HttpResponseMessage result</returns>
        public static async Task<string> HttpClientGetAsync(string webApiUrl)
        {
            string responseBody = null;

            using (var client = new HttpClient())
            {
                client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
                HttpResponseMessage response = await client.GetAsync(webApiUrl);

                if (!response.IsSuccessStatusCode)
                    throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
                else
                    responseBody = response.Content.ReadAsStringAsync().Result;
            }

            return responseBody;
        }

        /// <summary>
        /// Calls a Web API URL using HttpClient PostAsync.
        /// </summary>
        /// <param name="webApiUrl">Web API URL</param>
        /// <param name="serializedModel">Serialized json data that will be passed via the PostAsync call</param>
        /// <returns>Returns HttpResponseMessage</returns>
        public static async Task<HttpResponseMessage> HttpClientPostAsync(string webApiUrl, string serializedModel)
        {
            HttpResponseMessage response = null;

            using (var client = new HttpClient())
            {
                client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
                response = await client.PostAsync(webApiUrl, new StringContent(serializedModel, Encoding.UTF8, "application/json"));

                if (!response.IsSuccessStatusCode)
                    throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
            }

            return response;
        }

        /// <summary>
        /// Calls a Web API URL using HttpClient DeleteAsync.
        /// </summary>
        /// <param name="webApiUrl">Web API URL</param>
        /// <returns>Returns HttpResponseMessage</returns>
        public static async Task<HttpResponseMessage> HttpClientDeleteAsync(string webApiUrl)
        {
            HttpResponseMessage response = null;

            using (var client = new HttpClient())
            {
                client.BaseAddress = new Uri(Functions.GetWebApiBaseAddress());
                response = await client.DeleteAsync(webApiUrl);

                if (!response.IsSuccessStatusCode)
                    throw new Exception("Error Status Code: " + response.StatusCode.ToString() + " Error Reason: " + response.ReasonPhrase + " Error Message: " + response.RequestMessage.ToString());
            }

            return response;
        }
    }
}
